package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.client.TranServiceClient;
import com.diagbot.client.bean.HosCodeVO;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.dto.LisResult;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.enums.FeatureTypeEnum;
import com.diagbot.enums.LisSourceEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.SearchVo;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/7 16:12
 */
@Component
public class PushFacade {
    @Autowired
    private ClinicalFacade clinicalFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;
    @Autowired
    private TranServiceClient tranServiceClient;
    @Autowired
    DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    /**
     * 推理接口
     *
     * @param searchVo
     * @return ResponseData
     */
    public PushDTO pushInner(SearchVo searchVo) {
        PushDTO pushDTO = new PushDTO();
        //是否对接
        Boolean isConnect = false;
        if (StringUtil.isNotBlank(searchVo.getHosCode())) {
            HosCodeVO hosCodeVO = new HosCodeVO();
            hosCodeVO.setHosCode(searchVo.getHosCode());
            HospitalSetVO hospitalSetVO = new HospitalSetVO();
            hospitalSetVO.setHospitalCode(searchVo.getHosCode());
            hospitalSetVO.setCode("connect");
            if (searchVo.getSysType() != null) {
                hospitalSetVO.setSysType(searchVo.getSysType());
            } else {
                hospitalSetVO.setSysType(SysTypeEnum.AIPT_SERVICE.getKey());
            }
            RespDTO<List<SysSetInfoDTO>> sysSetInfoListRes = tranServiceClient.getSysSetInfoDatas(hospitalSetVO);
            if (RespDTOUtil.respIsOK(sysSetInfoListRes)) {
                List<SysSetInfoDTO> sysSetInfoList = sysSetInfoListRes.data;
                if (ListUtil.isNotEmpty(sysSetInfoList)) {
                    if (sysSetInfoList.get(0).getValue().equals(1)) {
                        isConnect = true;
                    } else {
                        isConnect = false;
                    }
                } else {
                    isConnect = false;
                }
            }
        }

        //化验转公表项处理
        if (isConnect) {
            List<LisResult> lisArr = searchVo.getLisArr();
            lisArr = addUniqueName(lisArr, searchVo.getHosCode());
            searchVo.setLisArr(lisArr);
        }
        ResponseData data = clinicalFacade.processClinicalData(searchVo);

        String featureType = searchVo.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("featureTypeSet", featureTypeSet);
            invokeParams.put("data", data);
            invokeParams.put("isConnect", isConnect);
            invokeParams.put("hosCode", searchVo.getHosCode());
            pushDTO
                    = dataBeanAggregateQueryFacade.get("setAll", invokeParams, PushDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return pushDTO;
    }

    /**
     * 获取管理评估大数据推理内容
     *
     * @param searchVo
     * @return
     */
    public Map<String, JSONObject> getManagementEvaluationContent(SearchVo searchVo) {
        ResponseData data = clinicalFacade.processClinicalData(searchVo);

        String featureType = searchVo.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_ManagementEvaluation.getKey()))) {
            return data.getManagementEvaluation();
        } else {
            return null;
        }
    }

    /**
     * 获取量表推理内容
     *
     * @param searchVo
     * @return
     */
    public Map<String, Object> scale(SearchVo searchVo) {
        Map<String, Object> data = clinicalFacade.scale(searchVo);
        return data;
    }

    /**
     * 获取治疗方案
     *
     * @param searchVo
     * @return
     */
    public Map<String, Object> getTreatment(SearchVo searchVo) {
        ResponseData data = clinicalFacade.processClinicalData(searchVo);
        Map<String, JSONObject> treat = data.getTreat();
        if (StringUtil.isBlank(searchVo.getDiseaseName())) {
            throw new CommonException(CommonErrorCode.PARAM_ERROR, "请输入需获取治疗方案的诊断名称");
        }
        Map<String, Object> treatmentMap
                = treatmentFacade.getTreatment(treat, searchVo.getDiseaseName(), searchVo.getDisType());
        return treatmentMap;
    }

    /**
     * 增加化验公表项
     *
     * @param lisResults
     * @param hosCode
     * @return
     */
    public List<LisResult> addUniqueName(List<LisResult> lisResults, String hosCode) {
        //化验项转公表内容,参数处理
        if (ListUtil.isNotEmpty(lisResults)) {
            List<String> mealNameList = lisResults
                    .stream()
                    .map(lisResult -> lisResult.getName()).collect(Collectors.toList());
            LisConfigVO lisConfigVO = new LisConfigVO();
            lisConfigVO.setHosCode(hosCode);
            lisConfigVO.setMealNameList(mealNameList);
            RespDTO<Map<String, Map<String, String>>> lisConfigRes
                    = tranServiceClient.getLisConfigByMealNameAndHosCode(lisConfigVO);
            Map<String, Map<String, String>> lisConfigMap = new LinkedHashMap<>();
            if (RespDTOUtil.respIsOK(lisConfigRes)) {
                lisConfigMap = lisConfigRes.data;
            }
            for (LisResult lisResult : lisResults) {
                if (!lisResult.getSource().equals(LisSourceEnum.Outer.getKey())) {
                    continue;
                }
                if (lisResult.getDetailName() == null) {
                    lisResult.setDetailName("");
                }
                if (lisConfigMap.get(lisResult.getName()) != null) {
                    lisResult.setUniqueName(lisConfigMap.get(lisResult.getName()).get(lisResult.getDetailName()));
                }
            }
        }
        return lisResults;
    }
}